// SPDX-FileCopyrightText: 2021 Florian Müllner <fmuellner@gnome.org>
// SPDX-License-Identifier: GPL-2.0-or-later

import Clutter from 'gi://Clutter';
import Gio from 'gi://Gio';
import GLib from 'gi://GLib';
import GObject from 'gi://GObject';
import Shell from 'gi://Shell';
import St from 'gi://St';

import {Extension} from 'resource:///org/gnome/shell/extensions/extension.js';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import {MonitorConstraint} from 'resource:///org/gnome/shell/ui/layout.js';

class ClassificationBanner extends Clutter.Actor {
    static {
        GObject.registerClass(this);
    }

    #topBanner;
    #bottomBanner;
    #monitorConstraint;
    #settings;

    constructor(index, settings) {
        const constraint = new MonitorConstraint({index});
        super({
            layout_manager: new Clutter.BinLayout(),
            constraints: constraint,
        });
        this.#monitorConstraint = constraint;

        Shell.util_set_hidden_from_pick(this, true);

        this.#settings = settings;

        this.#topBanner = new St.BoxLayout({
            style_class: 'classification-banner',
            x_expand: true,
            y_expand: true,
            y_align: Clutter.ActorAlign.START,
        });
        this.add_child(this.#topBanner);
        this.#settings.bind('top-banner',
            this.#topBanner, 'visible',
            Gio.SettingsBindFlags.GET);

        this.#bottomBanner = new St.BoxLayout({
            style_class: 'classification-banner',
            x_expand: true,
            y_expand: true,
            y_align: Clutter.ActorAlign.END,
        });
        this.add_child(this.#bottomBanner);
        this.#settings.bind('bottom-banner',
            this.#bottomBanner, 'visible',
            Gio.SettingsBindFlags.GET);

        for (const banner of [this.#topBanner, this.#bottomBanner]) {
            const label = new St.Label({
                style_class: 'classification-message',
                x_align: Clutter.ActorAlign.CENTER,
                x_expand: true,
            });
            banner.add_child(label);

            this.#settings.bind('message',
                label, 'text',
                Gio.SettingsBindFlags.GET);
        }

        const hostLabel = new St.Label({
            style_class: 'classification-system-info',
            text: GLib.get_host_name(),
        });
        this.#topBanner.insert_child_at_index(hostLabel, 0);
        this.#settings.bind('system-info',
            hostLabel, 'visible',
            Gio.SettingsBindFlags.GET);

        const userLabel = new St.Label({
            style_class: 'classification-system-info',
            text: GLib.get_user_name(),
        });
        this.#topBanner.add_child(userLabel);
        this.#settings.bind('system-info',
            userLabel, 'visible',
            Gio.SettingsBindFlags.GET);

        global.display.connectObject('in-fullscreen-changed',
            () => this.#updateMonitorConstraint(), this);
        this.#updateMonitorConstraint();

        this.#settings.connectObject(
            'changed::color', () => this.#updateStyles(),
            'changed::background-color', () => this.#updateStyles(),
            this);
        this.#updateStyles();
    }

    #getColorSetting(key) {
        const str = this.#settings.get_string(key);
        const [valid, color] = Clutter.Color.from_string(str);
        if (!valid)
            return '';
        const {red, green, blue, alpha} = color;
        return `${key}: rgba(${red},${green},${blue},${alpha / 255});`;
    }

    #updateMonitorConstraint() {
        const {index} = this.#monitorConstraint;
        this.#monitorConstraint.work_area =
            !global.display.get_monitor_in_fullscreen(index);
    }

    #updateStyles() {
        const bgStyle = this.#getColorSetting('background-color');
        const fgStyle = this.#getColorSetting('color');
        const style = `${bgStyle}${fgStyle}`;
        this.#topBanner.set({style});
        this.#bottomBanner.set({style});
    }
}

export default class ClassificationBannerExtension extends Extension {
    #banners = [];

    #updateMonitors() {
        const {monitors, panelBox, primaryIndex} = Main.layoutManager;
        if (monitors.length !== this.#banners.length) {
            this.#clearBanners();

            const settings = this.getSettings();
            for (let i = 0; i < monitors.length; i++) {
                const banner = new ClassificationBanner(i, settings);
                Main.uiGroup.add_child(banner);
                this.#banners.push(banner);
            }
        }

        const primaryBanner = this.#banners[primaryIndex];
        if (primaryBanner)
            Main.uiGroup.set_child_below_sibling(primaryBanner, panelBox);
    }

    #clearBanners() {
        this.#banners.forEach(b => b.destroy());
        this.#banners = [];
    }

    enable() {
        Main.layoutManager.connectObject('monitors-changed',
            () => this.#updateMonitors(), this);
        this.#updateMonitors();
    }

    disable() {
        Main.layoutManager.disconnectObject(this);
        this.#clearBanners();
    }
}
