import Adw from 'gi://Adw';
import Gdk from 'gi://Gdk';
import Gio from 'gi://Gio';
import GObject from 'gi://GObject';
import Gtk from 'gi://Gtk';

import {ExtensionPreferences, gettext as _} from 'resource:///org/gnome/Shell/Extensions/js/extensions/prefs.js';

class GenericPrefs extends Adw.PreferencesGroup {
    static {
        GObject.registerClass(this);
    }

    #actionGroup = new Gio.SimpleActionGroup();
    #settings;

    constructor(settings) {
        super();

        this.#settings = settings;
        this.insert_action_group('options', this.#actionGroup);

        this.#actionGroup.add_action(settings.create_action('top-banner'));
        this.#actionGroup.add_action(settings.create_action('bottom-banner'));
        this.#actionGroup.add_action(settings.create_action('system-info'));

        this.add(new Adw.SwitchRow({
            title: _('Top Banner'),
            action_name: 'options.top-banner',
        }));

        this.add(new Adw.SwitchRow({
            title: _('Bottom Banner'),
            action_name: 'options.bottom-banner',
        }));

        this.add(new Adw.SwitchRow({
            title: _('System Info'),
            action_name: 'options.system-info',
        }));
    }
}

class BannerPreset extends GObject.Object {
    static [GObject.properties] = {
        'message': GObject.ParamSpec.string(
            'message', 'message', 'message',
            GObject.ParamFlags.READWRITE,
            null),
        'color': GObject.ParamSpec.string(
            'color', 'color', 'color',
            GObject.ParamFlags.READWRITE,
            null),
        'background-color': GObject.ParamSpec.string(
            'background-color', 'background-color', 'background-color',
            GObject.ParamFlags.READWRITE,
            null),
    };

    static {
        GObject.registerClass(this);
    }
}

class AppearancePrefs extends Adw.PreferencesGroup {
    static {
        GObject.registerClass(this);
    }

    #settings;

    constructor(settings) {
        super();

        this.#settings = settings;

        const model = new Gio.ListStore({item_type: BannerPreset.$gtype});
        model.append(new BannerPreset({
            message: 'UNCLASSIFIED',
            color: '#fff',
            background_color: 'rgba(0, 122, 51, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'CONFIDENTIAL',
            color: '#fff',
            background_color: 'rgba(0, 51, 160, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'SECRET',
            color: '#fff',
            background_color: 'rgba(200, 16, 46, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'TOP SECRET',
            color: '#fff',
            background_color: 'rgba(255, 103, 31, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'TOP SECRET//SCI',
            color: '#000',
            background_color: 'rgba(247, 234, 72, 0.75)',
        }));

        let row, activatableWidget;
        row = this.#createPresetsRow(model);
        row.connect('notify::selected-item', comboRow => {
            const {message, color, backgroundColor} = comboRow.selected_item;
            this.#settings.set_string('message', message);
            this.#settings.set_string('color', color);
            this.#settings.set_string('background-color', backgroundColor);
        });
        this.add(row);

        activatableWidget = new Gtk.Entry({
            valign: Gtk.Align.CENTER,
        });
        this.#settings.bind('message',
            activatableWidget, 'text',
            Gio.SettingsBindFlags.DEFAULT);
        row = new Adw.ActionRow({title: _('Message'), activatableWidget});
        row.add_suffix(activatableWidget);
        this.add(row);

        activatableWidget = this.#createColorButton('background-color', {
            use_alpha: true,
        });
        row = new Adw.ActionRow({title: _('Background color'), activatableWidget});
        row.add_suffix(activatableWidget);
        this.add(row);

        activatableWidget = this.#createColorButton('color');
        row = new Adw.ActionRow({title: _('Text color'), activatableWidget});
        row.add_suffix(activatableWidget);
        this.add(row);
    }

    #createPresetsRow(model) {
        const listFactory = new Gtk.SignalListItemFactory();
        listFactory.connect('setup',
            (f, item) => item.set_child(new Gtk.Label()));
        listFactory.connect('bind', (f, listItem) => {
            const {child, item} = listItem;

            const provider = new Gtk.CssProvider();
            provider.load_from_data(`* {
                border-radius: 99px;
                padding: 6px;
                color: ${item.color};
                background-color: ${item.background_color};
            }`, -1);
            child.get_style_context().add_provider(provider,
                Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
            child.label = item.message;
        });

        return new Adw.ComboRow({
            title: _('Presets'),
            model,
            listFactory,
            expression: Gtk.ConstantExpression.new_for_value(''),
        });
    }

    #createColorButton(key, params = {}) {
        const rgba = new Gdk.RGBA();
        rgba.parse(this.#settings.get_string(key));

        const button = new Gtk.ColorButton({
            ...params,
            rgba,
            valign: Gtk.Align.CENTER,
        });
        this.#settings.connect(`changed::${key}`, () => {
            const newRgba = new Gdk.RGBA();
            newRgba.parse(this.#settings.get_string(key));
            if (!newRgba.equal(button.rgba))
                button.set({rgba: newRgba});
        });
        button.connect('notify::rgba',
            () => this.#settings.set_string(key, button.rgba.to_string()));
        return button;
    }
}

export default class ClassificationPrefs extends ExtensionPreferences {
    getPreferencesWidget() {
        const page = new Adw.PreferencesPage();
        page.add(new AppearancePrefs(this.getSettings()));
        page.add(new GenericPrefs(this.getSettings()));
        return page;
    }
}
